<?php

/**
 * @author      Rein de Vries <support@reinos.nl>
 * @link        https://addons.reinos.nl
 * @copyright   Copyright (c) 2020 Reinos.nl Internet Media
 * @license     https://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2017. Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

namespace Reinos\MapsMigration\Service;

require_once PATH_THIRD.'reinos_maps_migration/config.php';

class Migration
{
    public function __construct(){}

    public function has_backup_data()
    {
        //max time
        set_time_limit(0);

        if(ee('Model')->get('reinos_maps_migration:MigrateField')->count() > 0)
        {
            return 'true';
        }

        return 'false';

    }

    public function save_fields()
    {
        //max time
        set_time_limit(0);

        $q = ee()->db
            ->where('field_type', 'gmaps_fieldtype')
            ->from('channel_fields')
            ->get();

        if($q->num_rows() == 0) {
            return 'No old Gmaps fields found';
        }

        foreach($q->result() as $field)
        {
            ee('Model')->get('reinos_maps_migration:MigrateField')->filter('field_id', $field->field_id)->delete();
            ee('Model')->make('reinos_maps_migration:MigrateField')->set(array(
                'field_id' => $field->field_id,
                'data' => $field->field_settings,
            ))->save();
        }

        return 'All old Gmaps field prepared for migration';
    }

    public function save_entries()
    {
        //max time
        @ini_set('memory_limit','512M');
        set_time_limit(0);

        foreach( ee('Model')->get('reinos_maps_migration:MigrateField')->all() as $current_fields)
        {
            //get the data from the channnel data field
            $data = ee('Model')->get('ChannelEntry');

            //any result?
            if($data->count() > 0)
            {
                foreach($data->all() as $row)
                {
                    $field_data_raw = $row->{'field_id_'.$current_fields->field_id};

                    if($field_data_raw != '')
                    {
                        //backup data
                        ee('Model')->get('reinos_maps_migration:MigrateEntry')->filter('entry_id', $row->entry_id)->filter('field_id', $current_fields->field_id)->delete();
                        ee('Model')->make('reinos_maps_migration:MigrateEntry')->set(array(
                            'entry_id' => $row->entry_id,
                            'field_id' => $current_fields->field_id,
                            'data' => $field_data_raw,
                        ))->save();
                    }
                }
            }

            echo 'all entries for field ID: ' .$current_fields->field_id . " has been prepared for migration\n";
        }
    }

    public function migrate_fields()
    {
        //max time
        set_time_limit(0);
        @ini_set('memory_limit','512M');

        foreach( ee('Model')->get('reinos_maps_migration:MigrateField')->all() as $field)
        {
            $oldmap_data = unserialize(base64_decode($field->data));

            //build the new array
            $new_settings = array (
                'maps_fieldtype_icon_dir' => str_replace('gmaps', 'maps_fieldtype', $oldmap_data['gmaps_fieldtype_icon_dir']),
                'maps_fieldtype_icon_url' => str_replace('gmaps', 'maps_fieldtype', $oldmap_data['gmaps_fieldtype_icon_url']),
                'maps_fieldtype_location' => $oldmap_data['gmaps_fieldtype_location'],
                'maps_fieldtype_height' => $oldmap_data['gmaps_fieldtype_height'],
                'maps_fieldtype_max_markers' => $oldmap_data['gmaps_fieldtype_max_markers'],
                'maps_fieldtype_zoom_level' => $oldmap_data['gmaps_fieldtype_zoom_level'],
                'maps_fieldtype_show_map_tools' => $oldmap_data['gmaps_fieldtype_show_map_tools'],
                'maps_fieldtype_show_search_tools' => $oldmap_data['gmaps_fieldtype_show_search_tools'],
                'maps_fieldtype_show_marker_icon' => $oldmap_data['gmaps_fieldtype_show_marker_icon'],
                'maps_fieldtype_show_circle_icon' => $oldmap_data['gmaps_fieldtype_show_circle_icon'],
                'maps_fieldtype_show_polygon_icon' => $oldmap_data['gmaps_fieldtype_show_polygon_icon'],
                'maps_fieldtype_show_polyline_icon' => $oldmap_data['gmaps_fieldtype_show_polyline_icon'],
                'maps_fieldtype_show_rectangle_icon' => $oldmap_data['gmaps_fieldtype_show_rectangle_icon'],
                'maps_fieldtype_show_geocode_latlng' => $oldmap_data['gmaps_fieldtype_show_geocode_latlng'],
                'maps_fieldtype_auto_center' => $oldmap_data['gmaps_fieldtype_auto_center'],
                'maps_fieldtype_geocode_latlng' => $oldmap_data['gmaps_fieldtype_geocode_latlng'],
                'field_wide' => $oldmap_data['field_wide']
            );

            //-------------------------------------------------------------------------------------------
            // Save the data
            //-------------------------------------------------------------------------------------------
            ee()->db
                ->where('field_type', 'gmaps_fieldtype')
                ->where('field_id', $field->field_id)
                ->update('channel_fields', array(
                    'field_settings' => base64_encode(serialize($new_settings)),
                    'field_type' => 'reinos_maps_fieldtype'
                ));
        }

        return 'All Old Gmaps Fields are migrated to the new Maps Fieldtype structure';
    }

    public function get_migration_entries()
    {
        //max time
        set_time_limit(0);

        $limit = ee()->input->get('limit');
        $offset = ee()->input->get('offset');
        return ee('Model')
            ->get('reinos_maps_migration:MigrateEntry')
            ->offset($offset)
            ->limit($limit)
            ->fields('entry_id', 'field_id')
            ->all()->toArray();
    }

    public function migrate_entries()
    {
        //max time
        set_time_limit(0);
        @ini_set('memory_limit','512M');

        $entries = ee()->input->post('entries');

        if(empty($entries) || !is_array($entries))
        {
            echo 'No entries to migrate';
            exit;
        }

        foreach($entries as $entry)
        {
            echo 'migrate entry ID: '.$entry['entry_id'].' <br>';

            $migrate_entry = ee('Model')->get('reinos_maps_migration:MigrateEntry')->filter('entry_id', $entry['entry_id'])->filter('field_id', $entry['field_id'])->first();

//            $new_settings = ee('Model')->get('reinos_maps_migration:MigrateField')->filter('field_id', $entry['field_id'])->first();
//            $new_settings_raw = $new_settings->data;
//            $field_settings = unserialize(base64_decode($new_settings_raw));

            $field_data_raw = $migrate_entry->data;
            $field_data = json_decode(base64_decode($field_data_raw));

            if($field_data_raw != '')
            {
                //if object, then set it back to array
                if(is_object($field_data))
                {
                    $field_data = array($field_data->{0});
                }

                //get the data
                $field_data = $field_data[0];

                //set the new data
                $new_data = new \stdClass;
                $new_data->{'0'} =  (object)array(
                    'markers' => array(),
                    'polylines' => array(),
                    'polygons' => array(),
                    'circles' => array(),
                    'rectangles' => array(),
                    'map' => array(),
                );

                //set the map
                $new_data->{'0'}->map = (object)array(
                    'zoom' => $field_data->map->zoom_level,
                    'max_zoom' => $field_data->map->zoom_level,
                    'scroll_wheel' => $field_data->map->scroll_wheel,
                    'zoom_control' => $field_data->map->zoom_control,
                    'zoom_control_position' => $this->convert_location($field_data->map->zoom_control_position),
                    'scale_control' => true,
                    'scale_control_position' => 'bottomleft',
                    'overlay_html' => $field_data->map->google_overlay_html,
                    'overlay_position' => $this->convert_location($field_data->map->google_overlay_position),
                    'center' => $field_data->map->center,
                    'tiles' => $this->convert_tiles($field_data->map->map_type, $field_data->map->map_types),
                );

                //markers
                if(isset($field_data->markers) && is_array($field_data->markers) && count($field_data->markers) > 0)
                {
                    foreach($field_data->markers as $marker)
                    {
                        $new_data->{'0'}->markers[] = (object)array(
                            'lat' => $marker->lat,
                            'lng' => $marker->lng,
                            'tooltip' => $marker->title,
                            'icon' => str_replace('themes/user/gmaps/images', 'themes/user/reinos_maps_fieldtype/images', $marker->icon),
                            'content' => $marker->content,
                        );
                    }
                }

                //circles
                if(isset($field_data->circles) && is_array($field_data->circles) && count($field_data->circles) > 0)
                {
                    foreach($field_data->circles as $circle)
                    {
                        $new_data->{'0'}->circles[] = (object)array(
                            'color' => $circle->strokeColor,
                            'opacity' => $circle->strokeOpacity,
                            'weight' => $circle->strokeWeight,
                            'fillColor' => $circle->fillColor,
                            'fillOpacity' => $circle->fillOpacity,
                            'lat' => $circle->lat,
                            'lng' => $circle->lng,
                            'radius' => $circle->radius,
                        );
                    }
                }

                //rectangles
                if(isset($field_data->rectangles) && is_array($field_data->rectangles) && count($field_data->rectangles) > 0)
                {
                    foreach($field_data->rectangles as $rectangle)
                    {
                        $latlngs = explode(',', $rectangle->bounds);
                        $new_data->{'0'}->rectangles[] = (object)array(
                            'color' => $rectangle->strokeColor,
                            'opacity' => $rectangle->strokeOpacity,
                            'weight' => $rectangle->strokeWeight,
                            'fillColor' => $rectangle->fillColor,
                            'fillOpacity' => $rectangle->fillOpacity,
                            'latlngs' => array(
                                array(
                                    $latlngs[0],
                                    $latlngs[1]
                                ),
                                array(
                                    $latlngs[2],
                                    $latlngs[3]
                                )
                            )
                        );
                    }
                }

                //polylines
                if(isset($field_data->polylines) && is_array($field_data->polylines) && count($field_data->polylines) > 0)
                {
                    foreach($field_data->polylines as $polyline)
                    {
                        $new_data->{'0'}->polylines[] = (object)array(
                            'color' => $polyline->strokeColor,
                            'opacity' => $polyline->strokeOpacity,
                            'weight' => $polyline->strokeWeight,
                            'latlngs' => $polyline->path
                        );
                    }
                }

                //polygons
                if(isset($field_data->polygons) && is_array($field_data->polygons) && count($field_data->polygons) > 0)
                {
                    foreach($field_data->polygons as $polygon)
                    {
                        $new_data->{'0'}->polygons[] = (object)array(
                            'color' => $polygon->strokeColor,
                            'opacity' => $polygon->strokeOpacity,
                            'weight' => $polygon->strokeWeight,
                            'fillColor' => $polygon->fillColor,
                            'fillOpacity' => $polygon->fillOpacity,
                            'latlngs' => $polygon->paths
                        );
                    }
                }

                //-------------------------------------------------------------------------------------------
                // Save the data
                //-------------------------------------------------------------------------------------------
                echo 'start saving <br>';
                $update_entry = ee('Model')->get('ChannelEntry')->filter('entry_id', $entry['entry_id'])->first();
                $update_entry->set(array(
                    'field_id_'.$entry['field_id'] => base64_encode(json_encode($new_data))
                ));
                $update_entry->save();
                echo 'saved <br>';
            }
        }

        echo 'true';
    }

    public function convert_tiles($map_type = '', $map_types = [])
    {

        $tiles = [];

        $tiles[] = $this->convert_tile($map_type);

        if(!empty($map_types))
        {
            foreach($map_types as $type)
            {
                $tiles[] = $this->convert_tile($type);
            }
        }

        return $tiles;
    }

    public function convert_tile($tile)
    {
        switch($tile)
        {
            case 'roadmap':
                return 'google_maps_roadmap';
                break;
            case 'hybrid':
                return 'google_maps_hybrid';
                break;
            case 'satellite':
                return 'google_maps_satellite';
                break;
            case 'terrain':
                return 'google_maps_terrain';
                break;
            default:
                return $tile;
        }
    }

    public function convert_location($location = '', $migration = true)
    {
        $new_location = '';

        //set a default value when migration
        if($migration)
        {
            $new_location = 'topleft';
        }

        switch($location)
        {
            case 'LEFT_TOP':
            case 'TOP_LEFT':
                $new_location = 'topleft';
                break;

            case 'TOP_RIGHT':
            case 'RIGHT_TOP':
            case 'TOP_CENTER':
                $new_location = 'topright';
                break;

            case 'LEFT_BOTTOM':
            case 'BOTTOM_LEFT':
            case 'LEFT_CENTER':
            case 'BOTTOM_CENTER':
                $new_location = 'bottomleft';
                break;

            case 'BOTTOM_RIGHT':
            case 'RIGHT_BOTTOM':
            case 'RIGHT_CENTER':
                $new_location = 'bottomright';
                break;
        }
        return $new_location;
    }
}
