<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/**
 * @author		Rein de Vries <support@reinos.nl>
 * @link		https://addons.reinos.nl
 * @copyright 	Copyright (c) 2020 Reinos.nl Internet Media
 * @license     https://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2020 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */


/**
 * Include the config file
 */
require_once PATH_THIRD.'reinos_maps_migration/config.php';

class Reinos_maps_migration_mcp {

	public $return_data;
	public $settings;


	/**
	 * Constructor
	 */
	public function __construct(){}

	// ----------------------------------------------------------------

	/**
	 * Index Function
	 *
	 * @return 	void
	 */
	public function index()
	{
		//load the view
		return $this->migrate_gmaps();
	}

	// ----------------------------------------------------------------

	/**
	 * Overview Function
	 *
	 * @return 	void
	 */
	public function migrate_gmaps()
	{
        if(isset($_GET['action']))
        {
            switch($_GET['action'])
            {
                case 'has_backup_data':
                    echo ee(REINOS_MAPS_MIGRATION_SERVICE_NAME.':Migration')->has_backup_data();
                    break;

                case 'save_fields':
                    echo ee(REINOS_MAPS_MIGRATION_SERVICE_NAME.':Migration')->save_fields();
                    break;

                case 'save_entries':
                    ee(REINOS_MAPS_MIGRATION_SERVICE_NAME.':Migration')->save_entries();
                    break;

                case 'migrate_entries':
                    ee(REINOS_MAPS_MIGRATION_SERVICE_NAME.':Migration')->migrate_entries();
                    break;

                case 'migrate_fields':
                    ee(REINOS_MAPS_MIGRATION_SERVICE_NAME.':Migration')->migrate_fields();
                    break;

                case 'get_migration_entries':
                    $entry_ids = ee(REINOS_MAPS_MIGRATION_SERVICE_NAME.':Migration')->get_migration_entries();
                    echo json_encode($entry_ids);
                    break;

            }

            exit;
        }

        //--------------------------
        //custom settings
        $title_page = 'Migrate GMAPS to MAPS';
        //end custom settings
        //-------------------------

        ee()->javascript->set_global(array(
            'maps_field_type_base_url' => ee('CP/URL', 'addons/settings/'.REINOS_MAPS_MIGRATION_MAP)
        ));

        ee()->cp->add_js_script(array(
                'ui'     => array('progressbar'),
            )
        );

        ee()->cp->add_to_foot('<script type="text/javascript">'.file_get_contents(__DIR__.'/assets/javascript/migration.js').'</script>');

        $vars['title_page'] = $title_page;

        return $this->wrapContent($vars, $title_page, 'migrate_gmaps');
	}

    private function wrapContent($vars, $title, $view) {
        // set the vars for the new viewbox
        $vars = [
            'body' => ee('View')->make(REINOS_MAPS_MIGRATION_MAP.':'.$view)->render($vars),
            'title' => $title,
        ];

        $templateName = ee(REINOS_MAPS_MIGRATION_MAP.':Version')->isGteEE6() ? 'wrap/content' : 'wrap/legacy/content';

        return $this->output($templateName, $vars, $title);
    }

	// ----------------------------------------------------------------

	private function output($template, $vars, $heading = '')
	{
		//support for sidebar?
		$sidebar = ee('CP/Sidebar')->make();
		$module_sidebar = $sidebar->addHeader(lang(REINOS_MAPS_MIGRATION_MAP.'_module_name'));

		//create a list
		$module_list = $module_sidebar->addBasicList();
		$module_list->addItem(lang(REINOS_MAPS_MIGRATION_MAP.'_gmaps'), ee('CP/URL', 'addons/settings/'.REINOS_MAPS_MIGRATION_MAP.'/migrate_gmaps'));

        return array(
			'body'       => ee('View')->make(REINOS_MAPS_MIGRATION_MAP.':'.$template)->render($vars),
			'heading'    => REINOS_MAPS_MIGRATION_TITLE.' - '.lang($heading)
		);
	}

	// ----------------------------------------------------------------


}
