<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/**
 * @author		Rein de Vries <support@reinos.nl>
 * @link		https://addons.reinos.nl
 * @copyright 	Copyright (c) 2020 Reinos.nl Internet Media
 * @license     https://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2020 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

/**
 * Include the config file
 */
require_once PATH_THIRD.'reinos_maps_migration/config.php';

class Reinos_maps_migration_upd {

	public $version = REINOS_MAPS_MIGRATION_VERSION;

	/**
	 * Constructor
	 */
	public function __construct()
    {
	    //load the classes
        ee()->load->dbforge();

        //require the settings
        require PATH_THIRD.REINOS_MAPS_MIGRATION_MAP.'/settings.php';
    }

	// ----------------------------------------------------------------

	/**
	 * Installation Method
	 *
	 * @return 	boolean 	TRUE
	 */
	public function install()
	{
	    //set the module data
		$mod_data = array(
			'module_name'			=> REINOS_MAPS_MIGRATION_CLASS,
			'module_version'		=> REINOS_MAPS_MIGRATION_VERSION,
			'has_cp_backend'		=> "y",
			'has_publish_fields'	=> 'n'
		);

        //insert the module
        ee()->db->insert('modules', $mod_data);

        $this->install_tables();

		return TRUE;
	}

	// ----------------------------------------------------------------

	/**
	 * Uninstall
	 *
	 * @return 	boolean 	TRUE
	 */
	public function uninstall()
	{
		//delete the module
		ee()->db->where('module_name', REINOS_MAPS_MIGRATION_CLASS);
		ee()->db->delete('modules');

		$this->uninstall_tables();

		return TRUE;
	}

	// ----------------------------------------------------------------

    /**
     * Module Updater
     *
     * @param string $current
     * @return bool TRUE
     */
	public function update($current = '')
	{
		return true;
	}

    /**
     * Add the tables for the module
     *
     * @param string $table_name
     */
    public function install_tables($table_name = '')
    {
        foreach($this->mysql_table_data as $table => $data)
        {
            if($table_name != '' && $table != $table_name)
            {
                continue;
            }

            //table exists?
            if(!ee()->db->table_exists(REINOS_MAPS_MIGRATION_MAP.'_'.$table))
            {
                //add the fields
                if(isset($data['fields']))
                {
                    ee()->dbforge->add_field($data['fields']);
                }

                //add the keys
                if(isset($data['keys']))
                {
                    foreach($data['keys'] as $key)
                    {
                        $primary = isset($key[1]) && $key[1]? true : false;
                        ee()->dbforge->add_key($key[0], $primary);
                    }

                }

                ee()->dbforge->create_table(REINOS_MAPS_MIGRATION_MAP.'_'.$table, TRUE);
            }
        }
    }

    public function uninstall_tables()
    {
        //remove databases
        foreach($this->mysql_table_data as $table => $data)
        {
            ee()->dbforge->drop_table(REINOS_MAPS_MIGRATION_MAP.'_'.$table);
        }
    }
}
